@@ -1,3 +1,4 @@
+# vim:fileencoding=utf-8
 """Tests for urllib2-level functionality.
 
 This is urllib2's tests (most of which came from mechanize originally), plus
@@ -23,7 +24,7 @@ from mechanize import HTTPRedirectHandler, \
     HTTPCookieProcessor, HTTPRefererProcessor, \
     HTTPErrorProcessor, HTTPHandler
 from mechanize import OpenerDirector, build_opener, Request
-from mechanize._urllib2_fork import AbstractHTTPHandler
+from mechanize._urllib2_fork import AbstractHTTPHandler, normalize_url, AbstractBasicAuthHandler
 from mechanize._util import write_file
 
 import mechanize._response
@@ -68,6 +69,10 @@ class TrivialTests(mechanize._testcase.TestCase):
             self.assertEqual(
                 mechanize._urllib2_fork.parse_http_list(string), list)
 
+    def test_parse_authreq(self):
+        for bad in (",,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,",):
+            self.assertIsNone(AbstractBasicAuthHandler.rx.search(bad))
+
 
 def test_request_headers_dict():
     """
@@ -1364,7 +1369,7 @@ class HandlerTests(mechanize._testcase.TestCase):
             rp._sleep = st.sleep
             rp.http_response(
                 Request("http://example.com"),
-                test_response(headers=[("Refresh", header)]), )
+                test_response(headers=[("Refresh", header)], url="http://example.com/"), )
             self.assertEqual(expect_refresh, opener.called)
             st.verify()
 
@@ -1866,14 +1871,22 @@ class RequestTests(unittest.TestCase):
         self.assertEqual("POST", self.get.get_method())
 
     def test_get_full_url(self):
-        self.assertEqual("http://www.python.org/~jeremy/",
+        self.assertEqual("http://www.python.org/%7Ejeremy/",
                          self.get.get_full_url())
 
     def test_selector(self):
-        self.assertEqual("/~jeremy/", self.get.get_selector())
+        self.assertEqual("/%7Ejeremy/", self.get.get_selector())
         req = Request("http://www.python.org/")
         self.assertEqual("/", req.get_selector())
 
+    def test_normalize_url(self):
+        def t(x, expected=None):
+            self.assertEqual(normalize_url(x), expected or x)
+
+        t('https://simple.com/moo%7Ese')
+        t('https://ex.com/Spört', 'https://ex.com/Sp%C3%B6rt')
+        t('https://ex.com/Sp%C3%B6rt')
+
     def test_get_type(self):
         self.assertEqual("http", self.get.get_type())
 
